# load a simulated dataset of hospitalisation in different regions of France
# between March 1st 2020 to April 15th 2021

daily.numbers.hosp  = read.csv(file = 'data/Simulated_SIVIC_dataset.csv', sep = ';')
daily.numbers.hosp$date = as.Date(daily.numbers.hosp$date)
selected_dates = unique(daily.numbers.hosp$date)
all.age.classes = unique(daily.numbers.hosp$age)
Region.France.Metropolitaine = unique(daily.numbers.hosp$region)

# load 1000 values of the infection hospitalisation ratio for the different age classes
PH  =read.csv('data/IHR.csv', sep=';', header=FALSE, row.names =all.age.classes )


# list parameters 

Sensitivity= 0.85 # sensitivity of the serological assay 

## French population not in nursing homes -----
Region.France.Metropolitaine
pop.by.region  = read.csv(file = 'data/population_by_region_noEHPAD.csv',
                          sep=';',
                          header = TRUE,
                          row.names = Region.France.Metropolitaine,
                          col.names = all.age.classes)

names(pop.by.region) = all.age.classes

## aggregate population >70 yo 
pop.by.region_70 = pop.by.region
pop.by.region_70[,7]=rowSums(pop.by.region_70[,6:8])
pop.by.region_70=pop.by.region_70[,-c(7,8)]

## functions ---- 

mean.and.ci <-function(v){
  return( c(mean(v,na.rm = TRUE), as.numeric(quantile(v,probs = 0.025,na.rm = TRUE)), as.numeric(quantile(v,probs = 0.975,na.rm = TRUE))))
}
mean.and.sd <-function(v){
  m=mean(v,na.rm = TRUE)
  s=sd(v,na.rm = TRUE)
  return( c(m, m-s,m+s) )
} 

## functions for the delay distributions - 11 days for hospitalisation and 8 days for testing ----

X=seq(0,70,by=1)
k=11
theta=1
Y = pgamma(X,shape = k*theta, scale=1/theta)
delay.distribution=c(0,diff(Y))

k =8
theta=1
Y = pgamma(X,shape = 8, scale=1/theta)
delay.test.distribution=c(0,diff(Y))

delay <- function(n){
  d=0
  if(n<=0){
    d=0
  }
  if(n>=70){
    d=0
  }
  if(n>0 & n<70){
    d = delay.distribution[n]
  }
  return(d)
}

delay_test <- function(n){
  d=0
  if(n<=0){
    d=0
  }
  if(n>=70){
    d=0
  }
  if(n>0 & n<70){
    d = delay.test.distribution[n]
  }
  return(d)
} 

# Smoothing - Moving average 7 days
smooth.infected <- function(df2,N=7){
  ma <- function(x, n = N){stats::filter(x, rep(1 / n, n), sides = 2)}
  smoothed_ma<-ma(df2$infected)
  cor<-0
  # cherche le parametre de lissage qui permet de se rapprocher le plus d'une moyenne mobile de 7 jours
  for (k in seq(0.2,0.7,by=0.1)){
    smoothed_ss<-smooth.spline(x=df2$date,y=df2$infected,spar=k)$y
    cor.data<-cor.test(smoothed_ma,smoothed_ss)$estimate
    if (cor.data>cor){
      cor<-cor.data
      sparam<-k
    }
  }
  df2$smoothed<-smooth.spline(x=df2$date,y=df2$infected,spar=sparam)$y
  return(df2)
}

deconvolution <- function(Hospitalization, delay_fun=delay, offset=11){ 
  
  N1=14
  Hospitalization = c(rep(Hospitalization[1],N1), Hospitalization)
  
  L=length(Hospitalization)
  
  if(sum(Hospitalization)==0){
    lambda1 = rep(0,L)
  } 
  
  N0=sum(Hospitalization)
  
  if(N0>0){
    n=0
    KHI=2
    lambda0=Hospitalization
    lambda1=lambda0
    while(KHI >= 1 ){
      n=n+1
      Din = rep(0, L)# expected number of hospitalization to occur on day j
      for(i in 1:L){
        s=0
        for(j in seq(1,i)){
          s=s+delay_fun(i-j)*lambda0[j]
        }
        Din[i]=s
      }
      for(j in 1:(L-2)){
        qj =1
        SEQ = as.array(seq(-j+1,L-j))
        qj =sum(apply(SEQ,1,delay_fun))
        S=0
        for(i in seq(j+1,L)){
          S=S+delay_fun(i-j)*Hospitalization[i]/(Din[i]+0.00001) # I added a small number after Din to avoid dividing by 0 (happens when there is a long sequence of 0 in the hospitalization data)
        }
        lambda1[j] = lambda0[j]/qj*S
      }
      lambda0=lambda1
      Ein = rep(0, L)# expected number of hospitalization to occur on day j
      for(i in 1:L){
        s=0
        for(j in 1:i){
          s=s+delay_fun(i-j)*lambda0[j]
        }
        Ein[i]=s
      }
      w=which(Ein>0)
      KHI  = 1/N0*sum((Ein[w]-Din[w])^2/Ein[w])
    }
    lambda1[L]=0
  }
  lambda1=lambda1[-seq(1,N1)]
  return(lambda1)
}

# Compute the number of infection for different age class and region ----
number_infected <- function(dataframe,   age.cat,region, ph){
  
  if(!is.null(age.cat) & !is.null(region)  ){
    nb_filtered=  dataframe[which(dataframe$region == region & dataframe$age == age.cat ),]
  }
  if(is.null(age.cat) & !is.null(region)  ){
    nb_filtered=  dataframe[which(dataframe$region == region),]
  }
  if(!is.null(age.cat) & is.null(region)  ){
    nb_filtered=  dataframe[which( dataframe$age == age.cat ),]
    nb_filtered =  aggregate(nb_filtered$hosp_pred, by=list(date=nb_filtered$date), FUN=sum)
    colnames(nb_filtered)[2]="hosp_pred" # %>% rename(nb_filtered,   hosp_pred ="x")
  }
  if(is.null(age.cat) &  is.null(region)  ){
    nb_filtered=  dataframe 
    nb_filtered =  aggregate(nb_filtered$hosp_pred, by=list(date=nb_filtered$date), FUN=sum)
    colnames(nb_filtered)[2]="hosp_pred"  
    
  }
  w=which(rownames(ph)==age.cat)
  proba.hosp=ph[w]
  infected =  deconvolution(nb_filtered$hosp_pred)/proba.hosp
  return(infected)
}

number_infection_by_region <- function(dataframe, region=NULL, PH){ 
  
  reg.name=region
  if(is.null(region))
    reg.name='France Metropolitaine'
  df3=NULL
  ages = unique(dataframe$age)
  
  L = length(unique(dataframe$date))
  
  P = dim(PH)[2]
  for( p in 1:P){
    ph=  array(PH[,p],  dim=c(11,1))
    rownames(ph)=  all.age.classes
    infected=0
    for(age in ages){
      infected= infected+ number_infected(dataframe = daily.numbers.hosp,age.cat=age, region=region, ph=ph)
      number.infected = data.frame(date= selected_dates ,
                                   infected= infected,
                                   region=rep(reg.name,L),
                                   age=rep(age,L),
                                   PH=rep(p,L))
      
    }
    number.infected$infected  =  infected
    df3=rbind(df3, number.infected)
  }
  
  return(df3)
}

number_infection_by_age <- function(dataframe, region=NULL, PH, age=age){ 
  
  reg.name=region
  if(is.null(region))
    reg.name='France Metropolitaine'
  df3=NULL
  
  L = length(unique(dataframe$date))
  
  P = dim(PH)[2]
  for( p in 1:P){
    
    ph=  array(PH[,p],  dim=c(11,1))
    rownames(ph)=  all.age.classes
    print(p)
    
    infected=  number_infected(dataframe = daily.numbers.hosp,age.cat=age, region=region, ph=ph)
    number.infected = data.frame(date= selected_dates ,
                                 infected= infected ,
                                 region=rep(reg.name,L),
                                 age=rep(age,L),
                                 PH=rep(p,L))
    
    number.infected$infected  = infected 
    df3=rbind(df3, number.infected)
  }
  
  return(df3)
}
